# -*- coding:utf-8 -*- 
"""
投资参考数据接口 
Created on 2017/10/01
@author: Jimmy Liu
@group : waditu
@contact: jimmysoa@sina.cn
"""


def get_bond_info(code):
    pass



if __name__ == '__main__':
    pass